@@ -42,14 +42,6 @@ class QL(object):
       'Aggr': '%s',  # Placeholder to use formulas for aggregation.
       'Agg+': 'SUM(%s)',
       'Agg++': 'ARRAY_CONCAT_AGG(%s)',
-      # ArgMax and ArgMin return arg which achieves the max/min value.
-      'ArgMax': 'ARRAY_AGG({0}.arg order by {0}.value desc limit 1)[OFFSET(0)]',
-      'ArgMaxK':
-          'ARRAY_AGG({0}.arg order by {0}.value desc limit {1})',
-      'ArgMin': 'ARRAY_AGG({0}.arg order by {0}.value limit 1)[OFFSET(0)]',
-      'ArgMinK':
-          'ARRAY_AGG({0}.arg order by {0}.value limit {1})',
-      'Array': 'ARRAY_AGG({0}.value order by {0}.arg)',
       'Container': '%s',
       'Count': 'APPROX_COUNT_DISTINCT(%s)',
       'ExactCount': 'COUNT(DISTINCT %s)',
@@ -88,7 +80,6 @@ class QL(object):
       '<': '%s < %s',
       '>=': '%s >= %s',
       '>': '%s > %s',
-      '->': 'STRUCT(%s AS arg, %s as value)',
       '/': '(%s) / (%s)',
       '+': '(%s) + (%s)',
       '-': '(%s) - (%s)',
@@ -153,6 +144,7 @@ class QL(object):
     self.built_in_infix_operators = copy.deepcopy(
         self.BUILT_IN_INFIX_OPERATORS)
     self.built_in_infix_operators.update(self.dialect.InfixOperators())
+    self.CleanOperatorsAndFunctions()
     self.exception_maker = exception_maker
     self.debug_undefined_variables = False
     # We set convert_to_json to convert arguments of annotations to Python
@@ -162,6 +154,16 @@ class QL(object):
     self.flag_values = flag_values
     self.custom_udfs = custom_udfs or {}
 
+  def CleanOperatorsAndFunctions(self):
+    def CleanDictionary(d):
+      keys = list(d.keys())
+      for k in keys:
+        if d[k] is None:
+          del d[k]
+    for d in [self.built_in_infix_operators,
+              self.built_in_functions]:
+      CleanDictionary(self.built_in_infix_operators)
+
   @classmethod
   def BasisFunctions(cls):
     cls.InstallBulkFunctionsOfStandardSQL()
@@ -205,7 +207,7 @@ class QL(object):
                            'ParseTimestamp', 'FormatTimestamp',
                            'TimestampAddDays', 'Split', 'Element',
                            'Concat', 'DateAddDay', 'DateDiffDay',
-                           'ArgMaxK', 'ArgMinK', 'Join']
+                           'Join']
       if f in arity_2_functions:
         return (2, 2)
       return (1, 1)
@@ -232,7 +234,7 @@ class QL(object):
   def Subscript(self, record, subscript):
     if isinstance(subscript, int):
       subscript = 'col%d' % subscript
-    return '%s.%s' % (record, subscript)
+    return self.dialect.Subscript(record, subscript)
 
   def IntLiteral(self, literal):
     return str(literal['number'])
@@ -365,6 +367,43 @@ class QL(object):
       return self.ANALYTIC_FUNCTIONS[call['predicate_name']].format(
           aggregant, group_by, order_by, window_size)
 
+  def SubIfStruct(self, implication, subscript):
+    """Optimizing SQL for subscript of an 'if' statement.
+    
+    Args:
+      implication: Implication syntax tree.
+      subscript: Subscript string.
+      
+    Returns:
+      optimized SQL if all consequences are syntactic records, or None
+      otherwise.
+    """
+    def GetValueOfField(field_values, field):
+      for field_value in field_values:
+        if field_value['field'] == field:
+          return field_value['value']['expression']
+      raise self.exception_maker(
+          'Expected field %s missing in a record inside %s statement.' % (
+          color.Warn(subscript), color.Warn('if')))
+      assert False
+    all_records = all(
+        ('record' in if_then['consequence'])
+        for if_then in implication['if_then'])
+    if not (all_records and 'record' in implication['otherwise']):
+      return None
+    new_if_thens = []
+    for if_then in implication['if_then']:
+      new_if_then = copy.deepcopy(if_then)
+      consequence = GetValueOfField(
+          if_then['consequence']['record']['field_value'], subscript)
+      new_if_then['consequence'] = consequence
+      new_if_thens.append(new_if_then)
+    new_otherwise = GetValueOfField(
+        implication['otherwise']['record']['field_value'], subscript)
+    new_expr = {
+        'implication': {'if_then': new_if_thens, 'otherwise': new_otherwise}}
+    return self.ConvertToSql(new_expr)
+
   def ConvertToSql(self, expression):
     """Converting Logica expression into SQL."""
     # print('EXPR:', expression)
@@ -481,6 +520,12 @@ class QL(object):
           if f_v['field'] == subscript:
             # Optimizing and returning the field directly.
             return self.ConvertToSql(f_v['value']['expression'])
+      # Trying to optmize subscript of implication.
+      if 'implication' in sub['record']:
+        simplified_sub = self.SubIfStruct(sub['record']['implication'],
+                                          subscript)
+        if simplified_sub:
+          return simplified_sub
       # Couldn't optimize, just return the '.' expression.
       record = self.ConvertToSql(sub['record'])
       return self.Subscript(record, subscript)
