@@ -38,11 +38,19 @@ def RunAll():
   # Uncomment to test writing tables.
   # RunTest("ground_test")
   # RunTest("ground_psql_test")
+  # RunTest("closure_test")
+
   RunTest("psql_test")
+  RunTest("psql_arg_min_test")
+  RunTest("psql_arg_min_max_k_test")
+  RunTest("psql_pair_test")
+
   RunTest("sqlite_subquery_test")
 
   RunTest("sqlite_test")
 
+  RunTest("sub_if_test")
+
   RunTest(
       name="functor_chain_test",
       src="functor_chain_test.l",
@@ -242,3 +250,7 @@ def RunAll():
       golden="sql_string_table_test.txt",
       predicate="Test",
   )
+
+
+  RunTest("reachability_test")
+
