@@ -749,25 +749,30 @@ def ParseCall(s):
   idx = 0
   if not s:
     return None
-  for (idx, state, status) in Traverse(s):
-    assert status == 'OK'
-    if state == '(':
-      good_chars = (
-          set(string.ascii_letters) |
-          set(['@', '_', '.', '$', '{', '}', '+', '-', '`']) |
-          set(string.digits))
-      if ((idx > 0 and set(s[:idx]) <= good_chars) or
-          s[:idx] == '!' or
-          s[:idx] == '++?' or
-          idx >= 2 and s[0] == '`' and s[idx - 1] == '`'):
-        predicate = s[:idx]
-        break
-      else:
-        return None
-    if state and state != '{' and state[0] != '`':
-      return None
+  # Specialcasing -> operator for definition.
+  if s.startswith('->'):
+    idx = 2
+    predicate = '->'
   else:
-    return None
+    for (idx, state, status) in Traverse(s):
+      assert status == 'OK'
+      if state == '(':
+        good_chars = (
+            set(string.ascii_letters) |
+            set(['@', '_', '.', '$', '{', '}', '+', '-', '`']) |
+            set(string.digits))
+        if ((idx > 0 and set(s[:idx]) <= good_chars) or
+            s[:idx] == '!' or
+            s[:idx] == '++?' or
+            idx >= 2 and s[0] == '`' and s[idx - 1] == '`'):
+          predicate = s[:idx]
+          break
+        else:
+          return None
+      if state and state != '{' and state[0] != '`':
+        return None
+    else:
+      return None
   if (s[idx] == '(' and
       s[-1] == ')' and
       IsWhole(s[idx + 1:-1])):
@@ -911,12 +916,12 @@ def ParseNegation(s: HeritageAwareString):
 
 def ParseSubscript(s: HeritageAwareString):
   """Parsing subscript expression."""
-  str_path = Split(s, '.')
+  str_path = SplitRaw(s, '.')
   if len(str_path) >= 2:
     record_str = s[:(str_path[-2].stop - s.start)]
     record_str_doublecheck = '.'.join(str_path[:-1])
     assert record_str == record_str_doublecheck, 'This should not happen.'
-    record = ParseExpression(record_str)
+    record = ParseExpression(Strip(record_str))
     if not set(str_path[-1]) <= (set(string.ascii_lowercase) | set('_')
                                  | set(string.digits)):
       raise ParsingException('Subscript must be lowercase.', s)
